<?php
/****************************************************************************
Program: convertVocMapuche.php
Author:  Amos B. Batto, email: amosbatto AT yahoo DOT com
Project: ILLA, (www.illa-a.org) / SimiDic (www.simidic.org)
Created: 2014-01-01, (La Paz, Bolivia)
License: Public domain
*****************************************************************************/
$help = <<<HELP
convertVocMapuche.php converts the Vocabulario Mapuche-Castellano del Equipo de
Educación Mapuche Wixaleyiñ to be used in StarDict, GoldenDict and SimiDic.

To call this program:
   php convertVocMapuche.php VOC-MAPUCHE.html [OUTPUT]

For help:
   php convertVocMapuche -h 

Parameters:   
VOC-MAPUCHE.html The Vocabulario Mapuche-Castellano file saved as HTML.
OUTPUT           Prefix for output files. If not specified, then the same as
                 the filename of VOC-MAPUCHE.html without its extension. 

The script will strip out all HTML tags except <FONT COLOR="...">, <I>, and
<B> in the definitions. Then, it uses the text in the Mapuche-Castellano
section to create a Castellano-Mapuche dictionary. It then creates the 
following 4 StarDict TAB files:
- OUTPUT-map_es-html.tab (Mapuche-Castellano in HTML for GoldenDict & SimiDic)
- OUTPUT-map_es-html.tab (Mapuche-Castellano in Pango for StarDict)
- OUTPUT-es_map-html.tab (Castellano-Mapuche in HTML for GoldenDict & SimiDic)
- OUTPUT-es_map-html.tab (Castellano-Mapuche in Pango for StarDict)
Plus an HTML file containing Mapuche-Castellano & Castellano-Mapuche sections:
- OUTPUT-es_map.html

Finally the script calls StarDict’s tabfile utility to generate the binary 
StarDict files, which can be used in StarDict and GoldenDict. The script edits
the generated IFO files so StarDict and GoldenDict can display information 
about the Vocabulario.

After executing this script, run SimiDic-Builder and use The TAB files in 
HTML format to create the SimiDic Sqlite3 databases.

Requirements:
Assuming that using a Linux machine with UTF-8 default character set and PHP5 
installed. StarDict Tools need to be installed, which includes the tabfile 
utility.
 
HELP;

//configure PHP:
ini_set('max_execution_time', 0);  //set no max time for this program to run.
ini_set('memory_limit', '-1');  //set no memory limit
ini_set('register_argc_argv', true);
ini_set('auto_detect_line_endings', 'On');
setlocale(LC_ALL, 'es_BO');

$sPath = './';

if ($argc <= 1 or preg_match('@^(-h|--help|/h|help|ayuda)$@i', $argv[1]))
	exit($help);

if ($argc < 2)
	exit("Error: Too few parameters.\n\n" . $help);
elseif ($argc > 3)
	exit("Error: Too many parameters.\n\n" . $help);
	
$sFDic = $argv[1];

if ($argc > 2) {
	$sFOutput = $argv[2];
}
else {
	$aParts = pathinfo($sFDic);
	$sFOutput = $aParts['filename'];
}

if (false === ($sIn = file_get_contents($sFDic)))
	exit("Error: Unable to open file '$sFDic'.\n");
	
if (!($fMapEsPango = fopen($sFOutput . '-map_es-pango.tab', 'w')))
	exit("Error: Unable to open file '$sFOutput-map_es-pango.tab' for writing.\n");
	
if (!($fMapEsHtml = fopen($sFOutput . '-map_es-html.tab', 'w')))
	exit("Error: Unable to open file '$sFOutput-map_es-html.tab' for writing.\n");

//Get the text inside the <multicol>...</multicol> tags, which contains the Mapuche-Castellano section:
if (!preg_match('@<MULTICOL[^>]*>@mi', $sIn, $aVocMap, PREG_OFFSET_CAPTURE))
	exit("Error: Unable to extract the Mapuche-Castellano section.\n");
	
$sIn = mb_substr($sIn, $aVocMap[0][1] + mb_strlen($aVocMap[0][0]));
$nPosEnd = mb_stripos($sIn, '</MULTICOL>');
$sIn = mb_substr($sIn, 0, $nPosEnd);
$aParagraphs = preg_split('@(</P>)?\s*<P[^>]*>@im', $sIn, NULL, PREG_SPLIT_NO_EMPTY); 
$aEntriesMap = array(); //empty array to hold Mapuche entries in the dictionary

//loop through the paragraphs, pulling out the keys and definitions and inserting
//them in $aEntriesMap
for ($iPara = 0; $iPara < count($aParagraphs); $iPara++) {
	$sPara = trim($aParagraphs[$iPara]);
	$sPara = preg_replace("/ *\n */", ' ', $sPara);      //remove all hard returns
	$sPara = preg_replace('/ *\t+ */', ' ', $sPara);     //replace all tabs with spaces
	//remove all tags which aren't bold, italics or font tags:
	$sPara = strip_tags($sPara, '<i><b><em><strong><font>'); 

	//convert all tags to lowercase b/c Pango doesn't recognize uppercase tags 
	$sPara = preg_replace('@<(/?)(B|STRONG)>@i', '<\1b>', $sPara);
	$sPara = preg_replace('@<(/?)(I|EM)>@i', '<\1i>', $sPara);
	
	//eliminate all <font> tags except <font color="#008000"> (green) and 
	//<font color="#000080"> (blue) and convert to Pango (span fgcolor="#008000">)
	$sPara = preg_replace('@<FONT COLOR="#008000">(.*)</FONT>@iU', 
		'<span fgcolor="green">\1</span>', $sPara);
	$sPara = preg_replace('@<FONT COLOR="#000080">(.*)</FONT>@iU', 
		'<span fgcolor="blue">\1</span>', $sPara);
	$sPara = preg_replace('@</?FONT[^>]*>@i', '', $sPara);
	$sPara = str_replace('<span fgcolor="green"><i> </i></span>', ' ', $sPara);       
	$sPlainText = trim(strip_tags($sPara));

	//ignore lines which are empty or letter headings 	
	if (empty($sPlainText) or preg_match('/^[A-ZÑ]{1,3}$/', $sPlainText)) {
		continue;
	}
	
	//get key:
	if (!preg_match('/(.*) (1|deriv\.|adj\.|adv\.|aux\.|complem\.|conj\.|desin\.|fig\.|interj\.|interrog\.|' . 
		'modif\.|num\.|part\.|pospos\.|prep\.|pron\.|s\.|Sinón:|trans\.|v\.|voc\.|desinencia -ci,|el extremo,)/U', 
		$sPlainText, $aEntryParts))
	{
		print "Bad Entry without grammar: $sPlainText\n\n";
		continue;
	}
	$sKey = trim($aEntryParts[1]);
	
	//only two entries without an abbreviation to start the definition:	
	if ($sKey == 'ci' or  $sKey == 'af-') {
		preg_match('@( )@', $sPara, $aEntry, PREG_OFFSET_CAPTURE);
	}
	//find start of definition:
	elseif (!preg_match('@(<span fgcolor="green"><i>|<i><span fgcolor="green">|<b> *1)@', 
		$sPara, $aEntry, PREG_OFFSET_CAPTURE))
	{
		print "Bad Entry: $sPara\n\n";
		continue;
	}
	
	$sDefPango = trim(mb_substr($sPara, $aEntry[1][1]));
	$sKeyPango = trim(mb_substr($sPara, 0, $aEntry[1][1]));
	
	//eliminate any spaces between the first tag and the start of the definition>
	$sDefPango = preg_replace('@^((<i>|<b>|<span fgcolor="green">)+) +@', '\1', $sDefPango);

	//convert Pango to HTML:	 	
	$sDefHtml = str_replace('<span fgcolor=', '<font color=', $sDefPango);
	$sDefHtml = str_replace('</span>', '</font>', $sDefHtml);
	//Darken the green in Pango:
	$sDefPango = str_replace('<span fgcolor="green">', '<span fgcolor="#008000">', $sDefPango);
	
	$aEntriesMap[] = array('key' => $sKeyPango, 'def' => $sDefPango);
	
	fwrite($fMapEsPango, "$sKey\t$sDefPango\n");
	fwrite($fMapEsHtml,  "$sKey\t$sDefHtml\n");
}

fclose($fMapEsPango);
fclose($fMapEsHtml);

//create the Castellano-Pango section:
$aEntriesEs = array();

foreach ($aEntriesMap as $aEntryMap) {
	//$sDefPango = preg_replace('@(Expr:|Ej:)(.*)<span fgcolor="blue"><i>(.*)</i>< color$aEntryMap['def']
		
	$sDef = preg_replace('/ {2,}/', ' ', strip_tags($aEntryMap['def']));
	$sKey = strip_tags($aEntryMap['key']);
	$sGrammar = 'deriv\.[/ ]*|adj\.[/ ]*|adv\.[/ ]*|aux\.[/ ]*|complem\.[/ ]*|conj\.[/ ]*|desin\.[/ ]*|' .
		'fig\.[/ ]*|interj\.[/ ]*|interrog\.[/ ]*|modif\.[/ ]*|num\.[/ ]*|part\.[/ ]*|pospos\.[/ ]*|prep\.[/ ]*|' .
		'pron\.[/ ]*|s\.[/ ]*|trans\.[/ ]*|v\.trans\.[/ ]*|v\.trasl\.[/ ]*|v\.[/ ]*|voc\.[/ ]*';
	$aDefs = array();
	
	//if multiple definitions:
	if (preg_match_all('/\b[1-9] [^0-9]+/', $sDef, $aDefParts) and count($aDefParts[0]) >= 2) {
			
		//first check if there is a general grammatical part for all definitions:
		$sGrammarGeneral = '';  
		if (preg_match("@^($sGrammar)+@", $sDef, $aMatch)) {
			$sGrammarGeneral = $aMatch[0];
		}
			
		foreach ($aDefParts[0] as $sDefPart) {		
			//extract just the definition and grammatical part and exclude everything else
			if (preg_match('/^[1-9] (.+)(Ej:|Expr:|Sinón:|Ver:|Es mejor:|$)/U', $sDefPart, $aMatch)) {
				$sGrammarOnly = '';
				$sDefOnly     = $aMatch[1];
				
				//extract grammatical part, if exists
				if (preg_match("@(($sGrammar)+)(.*)$@", $aMatch[1], $aMatchGrammar)) {
					$sGrammarOnly = trim($aMatchGrammar[1]);
					$sDefOnly     = trim($aMatchGrammar[3]);
				}
				//insert general grammatical part if not found in this definition part
				if (empty($sGrammarOnly)) 
					$sGrammarOnly = $sGrammarGeneral;
				
				$aDefs[] = array('key' => $sKey, 'grammar' => $sGrammarOnly, 'def' => $sDefOnly);
			}
			else {
				print "Excluding in '$sKey': $sDefPart\n\n";
			}
		}
	}
	else { //if only one definition
		//extract just the definition and grammatical part and exclude everything else
		if (preg_match('/^(.+)(Ej:|Expr:|Sinón:|Ver:|Es mejor:|$)/U', $sDef, $aMatch)) {
			$sGrammarOnly = '';
			$sDefOnly     = $aMatch[1];
			
			//extract grammatical part, if exists
			if (preg_match("@(($sGrammar)+)(.*)$@", $aMatch[1], $aMatchGrammar)) {
				$sGrammarOnly = trim($aMatchGrammar[1]);
				$sDefOnly     = trim($aMatchGrammar[3]);
			}
			
			$aDefs[] = array('key' => $sKey, 'keyPango' => $aEntryMap['key'], 
				'grammar' => $sGrammarOnly, 'def' => $sDefOnly);			
		}
		else {
			print "Can't process '$sKey': $sDef\n\n";
		}
	}	
	
	foreach ($aDefs as $aDef) {
		//eliminate periods, commas and spaces at end of definitions: 
		$aDef['def'] = preg_replace('/[\.,; ]+$/', '', $aDef['def']);
		
		//move any text in beginning parentheses to the end of key:
		//ex: (trato recíproco) tía paterna -> tía paterna (trato recíproco)
		$sParentheses = '';
		if (preg_match('/^(\(.+\))(.+)$/', $aDef['def'], $aMatchParentheses)) {
			$aDef['def'] = trim($aMatchParentheses[2]);
			$sParentheses = $aMatchParentheses[1];
		}
			
		//if definition contains commas, then create en entry in the Castellano-Mapuche section for each one
		$aMultipleDefs = preg_split('/[,;] */', $aDef['def']);
		
		foreach ($aMultipleDefs as $sSingleDef) {
			//check if the key already exists. If so, get the next free key.			
			$i = 2;
			$keyToFind = $sSingleDef;

			for (; array_key_exists($keyToFind, $aEntriesEs); $i++) {
				$keyToFind = $sSingleDef . $i;
			}	
			if (!empty($aDef['key']) and !empty($sSingleDef) and $sSingleDef != 'etc') {
				$aEntriesEs[$keyToFind] = array(
					'key'     => trim($sSingleDef . ' ' . $sParentheses), 
					'grammar' => $aDef['grammar'], 
					'def'     => $aDef['key']
				);
			}
		}
	}	
}			 	

ksort($aEntriesEs, SORT_LOCALE_STRING);
$fEsMapPango = fopen($sFOutput . '-es_map-pango.tab', 'w');
$fEsMapHtml  = fopen($sFOutput . '-es_map-html.tab', 'w');

$fOutputHtml = fopen($sFOutput . '-es_map.html', 'w');
$header = <<<HEADER
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN">
<html>
<head>
	<META HTTP-EQUIV="CONTENT-TYPE" CONTENT="text/html; charset=utf-8">
	<title>Vocabulario Castellano-Mapuche</title>
	<meta name="author" content="Amos Batto" >	
	<META NAME="CREATED" CONTENT="20050413;3422600">
	<META NAME="CHANGEDBY" CONTENT="Amos Batto">
	<META NAME="CHANGED" CONTENT="20131213;2420600">
	<STYLE>
		<!-- 
		BODY,DIV,TABLE,THEAD,TBODY,TFOOT,TR,TH,TD,P {font-family:"serif";}
		 -->
	</STYLE>
</head>
<body>
<center>
<h1>Vocabulario Castellano-Mapuche</h1>
<p><big><b>del Equipo de Educación Mapuche Wixaleyiñ</b></big></p>
<p></p>
<p><b>
</center>
<a href="#Aes">A</a><br>
<a href="#Bes">B</a><br>
<a href="#Ces">C</a><br>
<a href="#Des">D</a><br>
<a href="#Ees">E</a><br>
<a href="#Fes">F</a><br>
<a href="#Hes">H</a><br>
<a href="#Ies">I</a><br>
<a href="#Jes">J</a><br>
<a href="#Les">L</a><br>
<a href="#LLes">LL</a><br>
<a href="#Mes">M</a><br>
<a href="#Nes">N</a><br>
<a href="#Oes">O</a><br>
<a href="#Pes">P</a><br>
<a href="#Qes">Q</a><br>
<a href="#Res">R</a><br>
<a href="#Ses">S</a><br>
<a href="#Tes">T</a><br>
<a href="#Ues">U</a><br>
<a href="#Ves">V</a><br>
<a href="#Yes">Y</a><br>
<a href="#Zes">Z</a>
</b></p>
<hr>
<h2 id="Aes"><b>A</b></h2>
		
HEADER;

fwrite($fOutputHtml, $header);
$aLettersEs = array('A', 'B', 'C', 'D', 'E', 'F', 'H', 'I', 'J', 'L', 'LL', 'M', 'N',
	 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'Y', 'Z');
$nCurLetter = 0;
foreach ($aEntriesEs as $aEntryEs) {
	$sDefPango = $aEntryEs['key'] . 
		"\t<span fgcolor=\"#008000\"><i>{$aEntryEs['grammar']}</i></span> {$aEntryEs['def']}.\n";
	$sDefHtml = str_replace('<span fgcolor="#008000">', '<font color="green">', $sDefPango);
	$sDefHtml = str_replace('</span>', '</font>', $sDefHtml);
	$sOutputHtml = "<p><b>{$aEntryEs['key']}</b> <font color=\"green\"><i>{$aEntryEs['grammar']}</i></font> {$aEntryEs['def']}.</p>\n";
		
	fwrite($fEsMapPango, $sDefPango);
	fwrite($fEsMapHtml,  $sDefHtml);
	
	$sLetter = strtoupper(mb_substr($aEntryEs['key'], 0, 1));
	if (mb_strlen($aEntryEs['key']) >= 2 and mb_substr($aEntryEs['key'], 0, 2) == 'll')
		$sLetter = 'LL';
	
	if ($sLetter != $aLettersEs[$nCurLetter] and $sLetter == $aLettersEs[$nCurLetter + 1]) {
		$nCurLetter++;
		fwrite($fOutputHtml, "<h2 id=\"{$sLetter}es\"><b>$sLetter</b></h2>\n");
	}
	
	fwrite($fOutputHtml, $sOutputHtml);
}	
		
fclose($fEsMapPango);
fclose($fEsMapHtml);
fwrite($fOutputHtml, '</body></html>');
fclose($fOutputHtml);

//create binary dictionaries for StarDict and GoldenDict:
createDic(
	$sBookname = 'Mapuche–Castellano (Educación Mapuche Wixaleyiñ)',
	$sDir      = 'map_es-wixaleyin-html',
	$sTabFile = $sFOutput . '-map_es-html.tab'
);
createDic(
	$sBookname = 'Mapuche–Castellano (Educación Mapuche Wixaleyiñ)',
	$sDir      = 'map_es-wixaleyin-pango',
	$sTabFile = $sFOutput . '-map_es-pango.tab'
);
createDic(
	$sBookname = 'Castellano–Mapuche (Educación Mapuche Wixaleyiñ)',
	$sDir      = 'es_map-wixaleyin-html',
	$sTabFile  = $sFOutput . '-es_map-html.tab'
);
createDic(
	$sBookname = 'Castellano–Mapuche (Educación Mapuche Wixaleyiñ)',
	$sDir      = 'es_map-wixaleyin-pango',
	$sTabFile  = $sFOutput . '-es_map-pango.tab'
);

return;

function createDic($sBookname, $sDir, $sTabFile) {
	$sAuthor = "Educación Mapuche Wixaleyiñ";
	$sDesc = "Equipo de Educación Mapuche Wixaleyiñ, Vocabulario Mapuche-Castellano, Argentina (2013), 45pp.";

	//run StarDict's tabfile to create the dictionaries for StarDict and GoldenDict
	system("tabfile ". $sTabFile);
	
	$aParts   = pathinfo($sTabFile);
	$sIfoFile = $aParts['filename'] . '.ifo';
	$sIfo = file_get_contents($sIfoFile);

	if (!preg_match('/wordcount=([0-9]+)/', $sIfo, $aMatch))
		exit("Error finding 'wordcount' in file '$sIfoFile'.\n"); 
	else
		$nWordCnt = $aMatch[1];
			
	if (!preg_match('/idxfilesize=([0-9]+)/', $sIfo, $aMatch))
		exit("Error finding 'idxfilesize' in file '$sIfoFile'.\n"); 
	else
		$nIdxSize = $aMatch[1];
			
	$sIfo = "StarDict's dict ifo file\n" .
		"version=2.4.2\n" .
		"wordcount=$nWordCnt\n" .
		"idxfilesize=$nIdxSize\n" .
		"bookname=$sBookname\n" .
		"description=$sDesc\n" .
		"author=$sAuthor\n" .
		"date=2013\n" .
		"website=http://www.illa-a.org/wp/diccionarios\n" .
		"email=amosbatto@yahoo.com\n" .
		"sametypesequence=g\n"; 
		
	file_put_contents($sIfoFile, $sIfo);

	//Create directories for the GoldenDict and StarDict files if they don't exist
	if (file_exists($sDir))
		system("rm $sDir/*");
	else
		mkdir($sDir, 0755);
	
	system("mv {$aParts['filename']}.*  $sDir");
	return;
}	 		
?>
